# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.classifications import universal_create_params
from ...types.classifications.universal_classification import UniversalClassification

__all__ = ["UniversalResource", "AsyncUniversalResource"]


class UniversalResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UniversalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return UniversalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UniversalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return UniversalResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["kanon-universal-classifier", "kanon-universal-classifier-mini"],
        query: str,
        texts: List[str],
        chunking_options: Optional[universal_create_params.ChunkingOptions] | NotGiven = NOT_GIVEN,
        is_iql: bool | NotGiven = NOT_GIVEN,
        scoring_method: Literal["auto", "chunk_max", "chunk_avg", "chunk_min"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UniversalClassification:
        """
        Classify the relevance of legal documents to a query with an Isaacus universal
        legal AI classifier.

        Args:
          model: The ID of the [model](https://docs.isaacus.com/models#universal-classification)
              to use for universal classification.

          query: The [Isaacus Query Language (IQL)](https://docs.isaacus.com/iql) query or, if
              IQL is disabled, the statement, to evaluate the texts against.

              The query must contain at least one non-whitespace character.

              Unlike the texts being classified, the query cannot be so long that it exceeds
              the maximum input length of the universal classifier.

          texts: The texts to classify.

              The texts must contain at least one non-whitespace character.

          chunking_options: Options for how to split text into smaller chunks.

          is_iql: Whether the query should be interpreted as an
              [IQL](https://docs.isaacus.com/iql) query or else as a statement.

          scoring_method: The method to use for producing an overall confidence score.

              `auto` is the default scoring method and is recommended for most use cases.
              Currently, it is equivalent to `chunk_max`. In the future, it will automatically
              select the best method based on the model and inputs.

              `chunk_max` uses the highest confidence score of all of the texts' chunks.

              `chunk_avg` averages the confidence scores of all of the texts' chunks.

              `chunk_min` uses the lowest confidence score of all of the texts' chunks.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/classifications/universal",
            body=maybe_transform(
                {
                    "model": model,
                    "query": query,
                    "texts": texts,
                    "chunking_options": chunking_options,
                    "is_iql": is_iql,
                    "scoring_method": scoring_method,
                },
                universal_create_params.UniversalCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UniversalClassification,
        )


class AsyncUniversalResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUniversalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUniversalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUniversalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/isaacus-dev/isaacus-python#with_streaming_response
        """
        return AsyncUniversalResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["kanon-universal-classifier", "kanon-universal-classifier-mini"],
        query: str,
        texts: List[str],
        chunking_options: Optional[universal_create_params.ChunkingOptions] | NotGiven = NOT_GIVEN,
        is_iql: bool | NotGiven = NOT_GIVEN,
        scoring_method: Literal["auto", "chunk_max", "chunk_avg", "chunk_min"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UniversalClassification:
        """
        Classify the relevance of legal documents to a query with an Isaacus universal
        legal AI classifier.

        Args:
          model: The ID of the [model](https://docs.isaacus.com/models#universal-classification)
              to use for universal classification.

          query: The [Isaacus Query Language (IQL)](https://docs.isaacus.com/iql) query or, if
              IQL is disabled, the statement, to evaluate the texts against.

              The query must contain at least one non-whitespace character.

              Unlike the texts being classified, the query cannot be so long that it exceeds
              the maximum input length of the universal classifier.

          texts: The texts to classify.

              The texts must contain at least one non-whitespace character.

          chunking_options: Options for how to split text into smaller chunks.

          is_iql: Whether the query should be interpreted as an
              [IQL](https://docs.isaacus.com/iql) query or else as a statement.

          scoring_method: The method to use for producing an overall confidence score.

              `auto` is the default scoring method and is recommended for most use cases.
              Currently, it is equivalent to `chunk_max`. In the future, it will automatically
              select the best method based on the model and inputs.

              `chunk_max` uses the highest confidence score of all of the texts' chunks.

              `chunk_avg` averages the confidence scores of all of the texts' chunks.

              `chunk_min` uses the lowest confidence score of all of the texts' chunks.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/classifications/universal",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "query": query,
                    "texts": texts,
                    "chunking_options": chunking_options,
                    "is_iql": is_iql,
                    "scoring_method": scoring_method,
                },
                universal_create_params.UniversalCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UniversalClassification,
        )


class UniversalResourceWithRawResponse:
    def __init__(self, universal: UniversalResource) -> None:
        self._universal = universal

        self.create = to_raw_response_wrapper(
            universal.create,
        )


class AsyncUniversalResourceWithRawResponse:
    def __init__(self, universal: AsyncUniversalResource) -> None:
        self._universal = universal

        self.create = async_to_raw_response_wrapper(
            universal.create,
        )


class UniversalResourceWithStreamingResponse:
    def __init__(self, universal: UniversalResource) -> None:
        self._universal = universal

        self.create = to_streamed_response_wrapper(
            universal.create,
        )


class AsyncUniversalResourceWithStreamingResponse:
    def __init__(self, universal: AsyncUniversalResource) -> None:
        self._universal = universal

        self.create = async_to_streamed_response_wrapper(
            universal.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from isaacus import Isaacus, AsyncIsaacus
from tests.utils import assert_matches_type
from isaacus.types import Reranking

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRerankings:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: Isaacus) -> None:
        reranking = client.rerankings.create(
            model="kanon-universal-classifier",
            query="What are the essential elements required to establish a negligence claim?",
            texts=[
                "To form a contract, there must be an offer, acceptance, consideration, and mutual intent to be bound.",
                "Criminal cases involve a completely different standard, requiring proof beyond a reasonable doubt.",
                "In a negligence claim, the plaintiff must prove duty, breach, causation, and damages.",
                "Negligence in tort law requires establishing a duty of care that the defendant owed to the plaintiff.",
                "The concept of negligence is central to tort law, with courts assessing whether a breach of duty caused harm.",
            ],
        )
        assert_matches_type(Reranking, reranking, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: Isaacus) -> None:
        reranking = client.rerankings.create(
            model="kanon-universal-classifier",
            query="What are the essential elements required to establish a negligence claim?",
            texts=[
                "To form a contract, there must be an offer, acceptance, consideration, and mutual intent to be bound.",
                "Criminal cases involve a completely different standard, requiring proof beyond a reasonable doubt.",
                "In a negligence claim, the plaintiff must prove duty, breach, causation, and damages.",
                "Negligence in tort law requires establishing a duty of care that the defendant owed to the plaintiff.",
                "The concept of negligence is central to tort law, with courts assessing whether a breach of duty caused harm.",
            ],
            chunking_options={
                "overlap_ratio": 0.1,
                "overlap_tokens": 0,
                "size": 512,
            },
            is_iql=False,
            scoring_method="auto",
            top_n=1,
        )
        assert_matches_type(Reranking, reranking, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: Isaacus) -> None:
        response = client.rerankings.with_raw_response.create(
            model="kanon-universal-classifier",
            query="What are the essential elements required to establish a negligence claim?",
            texts=[
                "To form a contract, there must be an offer, acceptance, consideration, and mutual intent to be bound.",
                "Criminal cases involve a completely different standard, requiring proof beyond a reasonable doubt.",
                "In a negligence claim, the plaintiff must prove duty, breach, causation, and damages.",
                "Negligence in tort law requires establishing a duty of care that the defendant owed to the plaintiff.",
                "The concept of negligence is central to tort law, with courts assessing whether a breach of duty caused harm.",
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        reranking = response.parse()
        assert_matches_type(Reranking, reranking, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: Isaacus) -> None:
        with client.rerankings.with_streaming_response.create(
            model="kanon-universal-classifier",
            query="What are the essential elements required to establish a negligence claim?",
            texts=[
                "To form a contract, there must be an offer, acceptance, consideration, and mutual intent to be bound.",
                "Criminal cases involve a completely different standard, requiring proof beyond a reasonable doubt.",
                "In a negligence claim, the plaintiff must prove duty, breach, causation, and damages.",
                "Negligence in tort law requires establishing a duty of care that the defendant owed to the plaintiff.",
                "The concept of negligence is central to tort law, with courts assessing whether a breach of duty caused harm.",
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            reranking = response.parse()
            assert_matches_type(Reranking, reranking, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRerankings:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncIsaacus) -> None:
        reranking = await async_client.rerankings.create(
            model="kanon-universal-classifier",
            query="What are the essential elements required to establish a negligence claim?",
            texts=[
                "To form a contract, there must be an offer, acceptance, consideration, and mutual intent to be bound.",
                "Criminal cases involve a completely different standard, requiring proof beyond a reasonable doubt.",
                "In a negligence claim, the plaintiff must prove duty, breach, causation, and damages.",
                "Negligence in tort law requires establishing a duty of care that the defendant owed to the plaintiff.",
                "The concept of negligence is central to tort law, with courts assessing whether a breach of duty caused harm.",
            ],
        )
        assert_matches_type(Reranking, reranking, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncIsaacus) -> None:
        reranking = await async_client.rerankings.create(
            model="kanon-universal-classifier",
            query="What are the essential elements required to establish a negligence claim?",
            texts=[
                "To form a contract, there must be an offer, acceptance, consideration, and mutual intent to be bound.",
                "Criminal cases involve a completely different standard, requiring proof beyond a reasonable doubt.",
                "In a negligence claim, the plaintiff must prove duty, breach, causation, and damages.",
                "Negligence in tort law requires establishing a duty of care that the defendant owed to the plaintiff.",
                "The concept of negligence is central to tort law, with courts assessing whether a breach of duty caused harm.",
            ],
            chunking_options={
                "overlap_ratio": 0.1,
                "overlap_tokens": 0,
                "size": 512,
            },
            is_iql=False,
            scoring_method="auto",
            top_n=1,
        )
        assert_matches_type(Reranking, reranking, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncIsaacus) -> None:
        response = await async_client.rerankings.with_raw_response.create(
            model="kanon-universal-classifier",
            query="What are the essential elements required to establish a negligence claim?",
            texts=[
                "To form a contract, there must be an offer, acceptance, consideration, and mutual intent to be bound.",
                "Criminal cases involve a completely different standard, requiring proof beyond a reasonable doubt.",
                "In a negligence claim, the plaintiff must prove duty, breach, causation, and damages.",
                "Negligence in tort law requires establishing a duty of care that the defendant owed to the plaintiff.",
                "The concept of negligence is central to tort law, with courts assessing whether a breach of duty caused harm.",
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        reranking = await response.parse()
        assert_matches_type(Reranking, reranking, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncIsaacus) -> None:
        async with async_client.rerankings.with_streaming_response.create(
            model="kanon-universal-classifier",
            query="What are the essential elements required to establish a negligence claim?",
            texts=[
                "To form a contract, there must be an offer, acceptance, consideration, and mutual intent to be bound.",
                "Criminal cases involve a completely different standard, requiring proof beyond a reasonable doubt.",
                "In a negligence claim, the plaintiff must prove duty, breach, causation, and damages.",
                "Negligence in tort law requires establishing a duty of care that the defendant owed to the plaintiff.",
                "The concept of negligence is central to tort law, with courts assessing whether a breach of duty caused harm.",
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            reranking = await response.parse()
            assert_matches_type(Reranking, reranking, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .universal_create_params import UniversalCreateParams as UniversalCreateParams
from .universal_classification import UniversalClassification as UniversalClassification



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .reranking import Reranking as Reranking
from .reranking_create_params import RerankingCreateParams as RerankingCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rerankings import (
    RerankingsResource,
    AsyncRerankingsResource,
    RerankingsResourceWithRawResponse,
    AsyncRerankingsResourceWithRawResponse,
    RerankingsResourceWithStreamingResponse,
    AsyncRerankingsResourceWithStreamingResponse,
)
from .classifications import (
    ClassificationsResource,
    AsyncClassificationsResource,
    ClassificationsResourceWithRawResponse,
    AsyncClassificationsResourceWithRawResponse,
    ClassificationsResourceWithStreamingResponse,
    AsyncClassificationsResourceWithStreamingResponse,
)

__all__ = [
    "ClassificationsResource",
    "AsyncClassificationsResource",
    "ClassificationsResourceWithRawResponse",
    "AsyncClassificationsResourceWithRawResponse",
    "ClassificationsResourceWithStreamingResponse",
    "AsyncClassificationsResourceWithStreamingResponse",
    "RerankingsResource",
    "AsyncRerankingsResource",
    "RerankingsResourceWithRawResponse",
    "AsyncRerankingsResourceWithRawResponse",
    "RerankingsResourceWithStreamingResponse",
    "AsyncRerankingsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["RerankingCreateParams", "ChunkingOptions"]


class RerankingCreateParams(TypedDict, total=False):
    model: Required[Literal["kanon-universal-classifier", "kanon-universal-classifier-mini"]]
    """
    The ID of the [model](https://docs.isaacus.com/models#reranking) to use for
    reranking.
    """

    query: Required[str]
    """The query to evaluate the relevance of the texts to.

    The query must contain at least one non-whitespace character.

    Unlike the texts being reranked, the query cannot be so long that it exceeds the
    maximum input length of the reranker.
    """

    texts: Required[List[str]]
    """The texts to rerank.

    There must be at least one text.

    The texts must contain at least one non-whitespace character.
    """

    chunking_options: Optional[ChunkingOptions]
    """Options for how to split text into smaller chunks."""

    is_iql: bool
    """
    Whether the query should be interpreted as an
    [Isaacus Query Language (IQL)](https://docs.isaacus.com/iql) query, which is not
    the case by default.

    If you allow untrusted users to construct their own queries, think carefully
    before enabling IQL since queries can be crafted to consume an excessively large
    amount of tokens.
    """

    scoring_method: Literal["auto", "chunk_max", "chunk_avg", "chunk_min"]
    """The method to use for producing an overall relevance score for a text.

    `auto` is the default scoring method and is recommended for most use cases.
    Currently, it is equivalent to `chunk_max`. In the future, it will automatically
    select the best method based on the model and inputs.

    `chunk_max` uses the highest relevance score of all of a text's chunks.

    `chunk_avg` averages the relevance scores of all of a text's chunks.

    `chunk_min` uses the lowest relevance score of all of a text's chunks.
    """

    top_n: Optional[int]
    """A whole number greater than or equal to 1."""


class ChunkingOptions(TypedDict, total=False):
    overlap_ratio: Optional[float]
    """A number greater than or equal to 0 and less than 1."""

    overlap_tokens: Optional[int]
    """A whole number greater than -1."""

    size: Optional[int]
    """A whole number greater than or equal to 1."""
